@@ -537,6 +537,7 @@ network_service_connect_sync (CamelNetworkService *service,
 	g_return_val_if_fail (connectable != NULL, NULL);
 
 	client = g_socket_client_new ();
+	g_socket_client_set_timeout (client, 90);
 
 	g_signal_connect (
 		client, "event",
@@ -591,7 +592,12 @@ network_service_new_connectable (CamelNetworkService *service)
 	port = camel_network_settings_get_port (network_settings);
 
 	if (host && *host)
-		connectable = g_network_address_new (host, port);
+		connectable = g_object_new (G_TYPE_NETWORK_ADDRESS,
+			"scheme", "socks",
+			"hostname", host,
+			"port", port,
+			NULL);
+
 
 	g_free (host);
 
@@ -713,7 +719,7 @@ camel_network_service_get_default_port (CamelNetworkService *service,
  * The returned #GSocketConnectable is referenced for thread-safety and
  * must be unreferenced with g_object_unref() when finished with it.
  *
- * Returns: a #GSocketConnectable
+ * Returns: (transfer full): a #GSocketConnectable
  *
  * Since: 3.8
  **/
@@ -832,7 +838,7 @@ camel_network_service_get_host_reachable (CamelNetworkService *service)
  * connection attempt is cancelled, the function sets @error and returns
  * %NULL.
  *
- * Returns: a #GIOStream, or %NULL
+ * Returns: (transfer full): a #GIOStream, or %NULL
  *
  * Since: 3.2
  **/
@@ -864,7 +870,7 @@ camel_network_service_connect_sync (CamelNetworkService *service,
  * This should typically be called after issuing a STARTTLS command
  * to a server to initiate a Transport Layer Security handshake.
  *
- * Returns: the new #GTlsClientConnection, or %NULL on error
+ * Returns: (transfer full): the new #GTlsClientConnection, or %NULL on error
  *
  * Since: 3.12
  **/
