@@ -33,6 +33,8 @@
 #include <glib-object.h>
 
 #include "e-source.h"
+#include "e-source-authentication.h"
+#include "e-source-credentials-provider-impl-google.h"
 #include "e-source-enumtypes.h"
 #include "e-source-registry.h"
 #include "camel/camel.h"
@@ -290,7 +292,8 @@ e_util_utf8_strstrcase (const gchar *haystack,
 		nuni[nlen++] = g_unichar_tolower (unival);
 	}
 	/* NULL means there was illegal utf-8 sequence */
-	if (!p) return NULL;
+	if (!p || !nlen)
+		return NULL;
 
 	o = haystack;
 	for (p = e_util_unicode_get_utf8 (o, &unival);
@@ -1149,7 +1152,7 @@ e_queue_transfer (GQueue *src_queue,
  *
  * Free the returned #GWeakRef with e_weak_ref_free().
  *
- * Returns: a new #GWeakRef
+ * Returns: (transfer full): a new #GWeakRef
  *
  * Since: 3.10
  **/
@@ -1329,7 +1332,7 @@ e_file_recursive_delete (GFile *file,
 /**
  * e_file_recursive_delete_finish:
  * @file: a #GFile to delete
- * @result: a #GAsyncResult
+ * @result: (transfer full): a #GAsyncResult
  * @error: return location for a #GError, or %NULL
  *
  * Finishes the operation started with e_file_recursive_delete().
@@ -1364,6 +1367,8 @@ e_file_recursive_delete_finish (GFile *file,
  * Thread safe variant of g_object_bind_property(). See its documentation
  * for more information on arguments and return value.
  *
+ * Returns: (transfer none):
+ *
  * Since: 3.16
  **/
 GBinding *
@@ -1378,10 +1383,27 @@ e_binding_bind_property (gpointer source,
 
 /**
  * e_binding_bind_property_full:
+ * @source: (type GObject.Object): the source #GObject
+ * @source_property: the property on @source to bind
+ * @target: (type GObject.Object): the target #GObject
+ * @target_property: the property on @target to bind
+ * @flags: flags to pass to #GBinding
+ * @transform_to: (scope notified) (allow-none): the transformation function
+ *   from the @source to the @target, or %NULL to use the default
+ * @transform_from: (scope notified) (allow-none): the transformation function
+ *   from the @target to the @source, or %NULL to use the default
+ * @user_data: custom data to be passed to the transformation functions,
+ *   or %NULL
+ * @notify: function to be called when disposing the binding, to free the
+ *   resources used by the transformation functions
  *
  * Thread safe variant of g_object_bind_property_full(). See its documentation
  * for more information on arguments and return value.
  *
+ * Return value: (transfer none): the #GBinding instance representing the
+ *   binding between the two #GObject instances. The binding is released
+ *   whenever the #GBinding reference count reaches zero.
+ *
  * Since: 3.16
  **/
 GBinding *
@@ -1400,11 +1422,24 @@ e_binding_bind_property_full (gpointer source,
 }
 
 /**
- * e_binding_bind_property_with_closures:
+ * e_binding_bind_property_with_closures: (rename-to e_binding_bind_property_full)
+ * @source: (type GObject.Object): the source #GObject
+ * @source_property: the property on @source to bind
+ * @target: (type GObject.Object): the target #GObject
+ * @target_property: the property on @target to bind
+ * @flags: flags to pass to #GBinding
+ * @transform_to: a #GClosure wrapping the transformation function
+ *   from the @source to the @target, or %NULL to use the default
+ * @transform_from: a #GClosure wrapping the transformation function
+ *   from the @target to the @source, or %NULL to use the default
  *
  * Thread safe variant of g_object_bind_property_with_closures(). See its
  * documentation for more information on arguments and return value.
  *
+ * Return value: (transfer none): the #GBinding instance representing the
+ *   binding between the two #GObject instances. The binding is released
+ *   whenever the #GBinding reference count reaches zero.
+ *
  * Since: 3.16
  **/
 GBinding *
@@ -2719,7 +2754,7 @@ e_source_registry_debug_print (const gchar *format,
 /**
  * e_type_traverse:
  * @parent_type: the root #GType to traverse from
- * @func: the function to call for each visited #GType
+ * @func: (scope call): the function to call for each visited #GType
  * @user_data: user data to pass to the function
  *
  * Calls @func for all instantiable subtypes of @parent_type.
@@ -2834,3 +2869,57 @@ e_util_get_source_full_name (ESourceRegistry *registry,
 
 	return g_string_free (fullname, FALSE);
 }
+
+
+/**
+ * e_util_get_source_oauth2_access_token_sync:
+ * @source: an #ESource
+ * @credentials: an ENamedParameters
+ * @out_access_token: (allow-none) (out): return location for the access token,
+ *                    or %NULL
+ * @out_expires_in_seconds: (allow-none) (out): return location for the token expiry,
+ *                  or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @error: return location for a #GError, or %NULL
+ *
+ * Obtains the OAuth 2.0 access token for @source along with its expiry
+ * in seconds from the current time (or 0 if unknown).
+ *
+ * Free the returned access token with g_free() when finished with it.
+ * If an error occurs, the function will set @error and return %FALSE.
+ *
+ * Returns: %TRUE on success, %FALSE on error
+ **/
+gboolean
+e_util_get_source_oauth2_access_token_sync (ESource *source,
+					    const ENamedParameters *credentials,
+					    gchar **out_access_token,
+					    gint *out_expires_in_seconds,
+					    GCancellable *cancellable,
+					    GError **error)
+{
+	gchar *auth_method = NULL;
+	gboolean success = FALSE;
+
+	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
+
+	if (e_source_has_extension (source, E_SOURCE_EXTENSION_AUTHENTICATION)) {
+		ESourceAuthentication *extension;
+
+		extension = e_source_get_extension (source, E_SOURCE_EXTENSION_AUTHENTICATION);
+		auth_method = e_source_authentication_dup_method (extension);
+	}
+
+	if (g_strcmp0 (auth_method, "OAuth2") == 0) {
+		success = e_source_get_oauth2_access_token_sync (
+			source, cancellable, out_access_token,
+			out_expires_in_seconds, error);
+	} else if (g_strcmp0 (auth_method, "Google") == 0) {
+		success = e_source_credentials_google_get_access_token_sync (
+			source, credentials, out_access_token, out_expires_in_seconds, cancellable, error);
+	}
+
+	g_free (auth_method);
+
+	return success;
+}
